package com.cultural.heritage.controller.other;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.CommonStringRequest;
import com.cultural.heritage.model.dto.banner.BannerAddRequest;
import com.cultural.heritage.model.dto.banner.BannerQueryRequest;
import com.cultural.heritage.model.dto.banner.BannerUpdateRequest;
import com.cultural.heritage.model.entity.Banner;
import com.cultural.heritage.model.vo.banner.BannerVO;
import com.cultural.heritage.service.common.CommonService;
import com.cultural.heritage.service.other.BannerService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/banner")
@Slf4j
@Tag(name = "轮播图管理")
public class BannerController {


    @Resource
    private BannerService bannerService;


    @Resource
    private CommonService commonService;


    /**
     * web端管理员添加轮播图
     * @param bannerAddRequest 轮播图添加请求体
     * @return  是否添加成功
     */
    @PostMapping("/add")
    @Operation(summary = "web端管理员添加轮播图", description = "参数：轮播图添加请求体，权限：管理员（boss, admin)，方法名：addBanner")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> addBanner(@RequestBody BannerAddRequest bannerAddRequest) {
        if (bannerAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Banner banner = new Banner();
        BeanUtils.copyProperties(bannerAddRequest, banner);
        bannerService.validBanner(banner, false);
        boolean result = bannerService.save(banner);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "轮播图添加失败");
        return ResultUtils.success(true);
    }



    /**
     * web端管理员删除轮播图
     * @param commonRequest 轮播图id
     * @return  是否成功删除
     */
    @PostMapping("/delete")
    @Operation(summary = "web端管理员删除轮播图", description = "参数：轮播图id，权限：管理员（boss, admin)，方法名：delBanner")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> delBanner(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        boolean result = bannerService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "轮播图删除失败");
        return ResultUtils.success(true);
    }




    /**
     * web端管理员更新轮播图
     * @param bannerUpdateRequest 轮播图更新请求体
     * @return 是否更新成功
     */
    @PostMapping("/update")
    @Operation(summary = "web端管理员更新轮播图", description = "参数：轮播图更新请求体，权限：管理员（boss, admin)，方法名：updateBanner")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateBanner(@RequestBody BannerUpdateRequest bannerUpdateRequest) {
        if (bannerUpdateRequest == null || bannerUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Banner banner = new Banner();
        BeanUtils.copyProperties(bannerUpdateRequest, banner);
        bannerService.validBanner(banner, true);
        boolean result = bannerService.updateById(banner);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "轮播图更新失败");
        return ResultUtils.success(true);
    }





    /**
     * Web端管理员分页查看轮播图
     * @return 轮播图列表
     */
    @PostMapping("/query/web")
    @Operation(summary = "Web端管理员分页查看轮播图", description = "参数：无，权限：管理员（boss, admin)，方法名:listBannerByPage")
    public BaseResponse<Page<BannerVO>> listBannerByPage(@RequestBody BannerQueryRequest bannerQueryRequest) {
        if (bannerQueryRequest == null) throw new BusinessException(ErrorCode.PARAMS_ERROR);
        long current = bannerQueryRequest.getCurrent();
        long pageSize = bannerQueryRequest.getPageSize();
        String type = bannerQueryRequest.getType();
        QueryWrapper<Banner> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(type), "type", type);
        Page<Banner> page = bannerService.page(new Page<>(current, pageSize),queryWrapper);
        List<Banner> bannerList = page.getRecords();
        List<BannerVO> bannerVOList = commonService.convertList(bannerList, BannerVO.class);
        Page<BannerVO> voPage = new Page<>();
        voPage.setRecords(bannerVOList);
        voPage.setPages(page.getPages());
        voPage.setCurrent(page.getCurrent());
        voPage.setTotal(page.getTotal());
        voPage.setSize(page.getSize());
        return ResultUtils.success(voPage);
    }



    /**
     * 小程序用户根据类型查看轮播图
     * @return 轮播图列表
     */
    @PostMapping("/query")
    @Operation(summary = "小程序用户根据类型查看轮播图", description = "参数：无，权限：所有人，方法名:queryBannerByType")
    public BaseResponse<List<BannerVO>> queryBannerByType(@RequestBody CommonStringRequest commonStringRequest) {
        if (commonStringRequest == null || StringUtils.isBlank(commonStringRequest.getType())) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        String type = commonStringRequest.getType();
        List<Banner> bannerList = commonService.findByFieldEqTargetField("type", type, bannerService);
        List<BannerVO> bannerVOList = commonService.convertList(bannerList, BannerVO.class);
        return ResultUtils.success(bannerVOList);
    }



}
